@@ -45,7 +45,7 @@ void HistoryManager::save()
 {
     QJsonDocument doc;
     QJsonArray array;
-    for (const auto &record : qAsConst(m_historyList)) {
+    for (const auto &record : std::as_const(m_historyList)) {
         array.append(record);
     }
     doc.setArray(array);
@@ -55,3 +55,5 @@ void HistoryManager::save()
     file.write(doc.toJson(QJsonDocument::Compact));
     qDebug() << "save" << file.fileName();
 }
+
+#include "moc_historymanager.cpp"
