@@ -19,6 +19,8 @@ class InputManager : public QObject
     Q_PROPERTY(bool moveFromResult READ moveFromResult NOTIFY resultChanged)
     Q_PROPERTY(int cursorPosition READ getCursorPosition WRITE setCursorPosition NOTIFY cursorPositionChanged)
     Q_PROPERTY(bool binaryMode READ binaryMode WRITE setBinaryMode)
+    Q_PROPERTY(bool canUndo READ canUndo NOTIFY canUndoChanged)
+    Q_PROPERTY(bool canRedo READ canRedo NOTIFY canRedoChanged)
 
 public:
     static InputManager *inst();
@@ -34,12 +36,18 @@ public:
     Q_INVOKABLE void append(const QString &subexpression);
     Q_INVOKABLE void backspace();
     Q_INVOKABLE void equal();
-    Q_INVOKABLE void clear();
+    Q_INVOKABLE void clear(bool save = true);
     Q_INVOKABLE void fromHistory(const QString &result);
+    void store();
+    Q_INVOKABLE void undo();
+    Q_INVOKABLE void redo();
+    bool canUndo();
+    bool canRedo();
     void setBinaryMode(bool active);
     bool binaryMode();
     QString formatNumbers(const QString &text);
     void addNumberSeparators(QString &number);
+    void calculate();
 
 Q_SIGNALS:
     void expressionChanged();
@@ -47,6 +55,8 @@ Q_SIGNALS:
     void binaryResultChanged();
     void hexResultChanged();
     void cursorPositionChanged();
+    void canUndoChanged();
+    void canRedoChanged();
 
 private:
     InputManager();
@@ -61,4 +71,6 @@ private:
     bool m_isBinaryMode = false; // Changes the parser based on this variable
     QString m_groupSeparator;
     QString m_decimalPoint;
+    std::vector<QString> m_undoStack;
+    size_t m_undoPos = 0;
 };
