@@ -34,7 +34,6 @@ int main(int argc, char *argv[])
 {
     QCommandLineParser parser;
 
-    QGuiApplication::setAttribute(Qt::AA_EnableHighDpiScaling);
 #ifdef Q_OS_ANDROID
     QGuiApplication app(argc, argv);
     QQuickStyle::setStyle(QStringLiteral("org.kde.breeze"));
@@ -51,7 +50,7 @@ int main(int argc, char *argv[])
     }
 #endif
     QQmlApplicationEngine engine;
-    KLocalizedString::setApplicationDomain("kalk");
+    KLocalizedString::setApplicationDomain(QByteArrayLiteral("kalk"));
     parser.addVersionOption();
     engine.rootContext()->setContextObject(new KLocalizedContext(&engine));
 
